import java.awt.geom.Rectangle2D;
import java.security.KeyStore;
import java.util.Enumeration;

import com.digisigner.api.PdfDocument;
import com.digisigner.api.SignatureAppearanceData;
import com.digisigner.api.SignatureRenderingModeEnum;
import com.digisigner.api.SigningData;
import com.digisigner.api.SignatureAppearanceData.SignaturePartData;


public class Examples {

	/**
	 * Creates signature with standard appearance.
	 */
	private static void standardSignature() {
		
		// specify document to sign
		PdfDocument pdfDocument = new PdfDocument("sample.pdf");
		
		// specify key in the key store
		SigningData signingData = SigningData.fromPkcs12File("keystore.pfx", "keystore_pass", "key_alias");
		
		// specify rectangle for the signature
		signingData.setFieldRectangle(new Rectangle2D.Float(275, 50, 300, 100));
		
		// sign document
		pdfDocument.signDocument(signingData, "sample_(signed_standard).pdf");		
	}
	
	/**
	 * Creates signature with an image on the left side and signature text on the right side.
	 * For the text a custom font is used.
	 */
	private static void signatureWithImageAndCustomFont() {
		
		// specify document to sign
		PdfDocument pdfDocument = new PdfDocument("sample.pdf");
		
		// specify key in the key store
		SigningData signingData = SigningData.fromPkcs12File("keystore.pfx", "keystore_pass", "key_alias");
		
		// specify rectangle for the signature
		signingData.setFieldRectangle(new Rectangle2D.Float(275, 50, 300, 100));
		
		// use image for the left part of the signature
		SignaturePartData signatureLeftPart = signingData.getAppearanceData().getLeftPartData();
		signatureLeftPart.setRenderingMode(SignatureRenderingModeEnum.GRAPHIC);
		signatureLeftPart.setGraphicFilename("signature.png");
		
		// use custom font for the right part of the signature
		SignaturePartData signatureRightPart = signingData.getAppearanceData().getRightPartData();
		signatureRightPart.setFontFilepath("font.ttf");		
			
		// sign document
		pdfDocument.signDocument(signingData, "sample_(signed_with_image_and_text).pdf");		
	}
	
	/**
	 * Creates signature, which appearance represents an image.  
	 */
	private static void signatureWithImage() {
		
		// specify document to sign
		PdfDocument pdfDocument = new PdfDocument("sample.pdf");
		
		// specify key in the key store
		SigningData signingData = SigningData.fromPkcs12File("keystore.pfx", "keystore_pass", "key_alias");
		
		// specify rectangle for the signature
		signingData.setFieldRectangle(new Rectangle2D.Float(325, 50, 200, 100));
		
		// use image for the left part of the signature
		SignatureAppearanceData appearanceData = signingData.getAppearanceData();
		SignaturePartData signatureLeftPart = appearanceData.getLeftPartData();
		signatureLeftPart.setRenderingMode(SignatureRenderingModeEnum.GRAPHIC);
		signatureLeftPart.setGraphicFilename("signature.png");
		
		// don't show right part
		SignaturePartData signatureRightPart = appearanceData.getRightPartData();
		signatureRightPart.setShowPart(false);		
		
		// sign document
		pdfDocument.signDocument(signingData, "sample_(signed_with_image).pdf");		
	}
	
	/**
	 * Creates signature, which appearance is a text, created with a custom font.
	 */
	private static void signatureWithText() {
		
		// specify document to sign
		PdfDocument pdfDocument = new PdfDocument("sample.pdf");
		
		// specify key in the key store
		SigningData signingData = SigningData.fromPkcs12File("keystore.pfx", "keystore_pass", "key_alias");
		
		// specify rectangle for the signature
		signingData.setFieldRectangle(new Rectangle2D.Float(300, 50, 275, 100));
		
		// specify reason and location
		signingData.setReason("I approve this invoice.");
		signingData.setLocation("Camden");
		
		// use image for the left part of the signature
		SignaturePartData signatureLeftPart = signingData.getAppearanceData().getLeftPartData();
		signatureLeftPart.setRenderingMode(SignatureRenderingModeEnum.TEXT);
		signatureLeftPart.setFontFilepath("font.ttf");
		
		// don't show right part
		SignaturePartData signatureRightPart = signingData.getAppearanceData().getRightPartData();
		signatureRightPart.setShowPart(false);				
		
		// sign document
		pdfDocument.signDocument(signingData, "sample_(signed_with_text).pdf");		
	}
	
	/**
	 * Creates signature using PKCS#11 token.
	 */
	private static void signatureWithToken() {
		
		// specify document to sign
		PdfDocument pdfDocument = new PdfDocument("sample.pdf");
		
		// specify info to find a key on the token
		String tokenLibraryFilename = "C:\\Windows\\System32\\cmP11.dll";  // path to token PKCS#11 library
		String tokenPin = "7777";  // token PIN
		// key alias on the token,
		// if null the first found key will be used
		String tokenKeyAlias = "Key Alias";
		SigningData signingData = SigningData.fromPkcs11Token(tokenLibraryFilename, tokenKeyAlias, tokenPin);
		
		// list available aliases
		listAvailableAliases(signingData);
		
		// specify rectangle for the signature
		signingData.setFieldRectangle(new Rectangle2D.Float(300, 50, 275, 100));
		
		// specify reason and location
		signingData.setReason("I approve this invoice.");
		signingData.setLocation("Camden");	
		
		// sign document
		pdfDocument.signDocument(signingData, "sample_(signed_with_token).pdf");		
	}

	/**
	 * Creates signature using PKCS#11 token.
	 */
	private static void signatureWithWindowsKeyStore() {
		
		// specify document to sign
		PdfDocument pdfDocument = new PdfDocument("sample.pdf");
		
		// specify key in the key store
		String keyAlias = "Key Alias";
		SigningData signingData = SigningData.fromWindowsKeyStore(keyAlias);
		
		// list available aliases
		listAvailableAliases(signingData);
		
		// specify rectangle for the signature
		signingData.setFieldRectangle(new Rectangle2D.Float(300, 50, 275, 100));
		
		// specify reason and location
		signingData.setReason("I approve this invoice.");
		signingData.setLocation("Camden");	
		
		// sign document
		pdfDocument.signDocument(signingData, "sample_(signed_with_windows_key_store).pdf");		
	}
	
	private static void listAvailableAliases(SigningData signingData) {
		System.out.println("Use one of the following aliases");
		
		KeyStore keyStore = signingData.getKeyStore();
		try {
			for (Enumeration<String> aliases = keyStore.aliases(); aliases.hasMoreElements();) {
				String keyAlias = aliases.nextElement();
		
				if (keyStore.entryInstanceOf(keyAlias, KeyStore.PrivateKeyEntry.class)) {
					System.out.println(keyAlias);
				}
			}
		}
		catch (Exception e) {
			// ignore
		}
	}
	
	public static void main(String[] args) {
		standardSignature();
		
		signatureWithImageAndCustomFont();
		
		signatureWithImage();
		
		signatureWithText();
		
		//signatureWithToken();
		
		//signatureWithWindowsKeyStore();
	}

}
